from django.core.exceptions import ObjectDoesNotExist
from django.shortcuts import render

from results.helpers import get_does_not_exist_model


def result_view(request, class_: str = 'info', title: str = None, message=None, form_errors=None):
    return render(request, 'results/result.html', {
        'class': class_,  # 'info' / 'success' / 'danger'
        'title': title or '出错了',
        'message': message or '未知错误',
        'errors': form_errors,
    })


def error_result(request, title=None, message=None, error=None, form=None):
    if error:
        if isinstance(error, ObjectDoesNotExist):
            if not title:
                model = get_does_not_exist_model(error)
                if model:
                    title = '找不到{}'.format(model._meta.verbose_name or model._meta.model_name)
                else:
                    title = '找不到对象'

    return result_view(request,
                       title=title or '出错了',
                       message=message,
                       class_="danger",
                       form_errors=form.errors if form else None)



from django.apps import apps


def get_does_not_exist_model(e):
    # 从ObjectDoesNotExist错误中查出找不到的model类型
    for model in apps.get_models():
        if model.DoesNotExist == e.__class__:
            return model
    return None



from functools import partial

from django.conf.urls import url
from django.core.exceptions import ObjectDoesNotExist
from django.views.generic import TemplateView

from results import views as results_views
from results.views import error_result
from sample import views as sample_views

urlpatterns = [
    url(r'^$', TemplateView.as_view(template_name='sample/index.html')),
    url(r'^form-error/$', sample_views.form_error_view, name='form_error'),

    url(r'^error-result/$', partial(error_result, title='出错标题', message='出错消息'), name='error_result'),
    url(r'^error-result/object-does-not-exist/$',
        partial(error_result, error=ObjectDoesNotExist()),
        name='error_result_object_does_not_exist'),

    url(r'^result_view/success/$',
        results_views.result_view,
        {"title": "success的标题", "message": "success的消息", "class_": 'success'}, name='result_view_success'),
    url(r'^result_view/danger/$',
        results_views.result_view,
        {"title": "danger的标题", "message": "danger的消息", "class_": 'danger'}, name='result_view_danger'),
    url(r'^result_view/info/$',
        results_views.result_view,
        {"title": "info的标题", "message": "info的消息", "class_": 'info'}, name='result_view_info'),
]



from django import forms
from django.shortcuts import render


def form_error_view(request):
    class TestForm(forms.Form):
        required_field = forms.CharField(required=True)

        invalid_field = forms.CharField(max_length=1)

        def clean(self):
            raise forms.ValidationError('non field error happened')

    form = TestForm(data={'invalid_field': 'xxxxx'})

    return render(request, 'results/result.html', context={
        'class': 'danger',
        'title': '这里有一个错误',
        'message': '不知道当讲不当讲，算了我还是讲了',
        'errors': form.errors,
    })
